@@ -10,5 +10,5 @@ Lambda Pipeline, a framework for writing clear, minimal Python FAAS
 from lpipe._version import __version__
 from lpipe.action import Action
 from lpipe.payload import Payload
-from lpipe.pipeline import process_event
+from lpipe.pipeline import EventSourceType, process_event
 from lpipe.queue import Queue, QueueType
